import 'dart:collection';

import 'package:meta/meta.dart';
import 'package:miodi/core/consts.dart';

import 'module.dart';
import 'providers.dart';
import '../error/errors.dart';

class MiodiComponent {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  final Map<String, Map<Type, AsyncProvider>> factoryAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> singletonAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> customAsyncConfiguration = HashMap();

  @mustCallSuper
  MiodiComponent(this.modules) {
    modules.forEach((module) {
      factoryConfiguration.addAll(module.factoryConfiguration);
      singletonConfiguration.addAll(module.singletonConfiguration);
      customConfiguration.addAll(module.customConfiguration);

      factoryAsyncConfiguration.addAll(module.factoryAsyncConfiguration);
      singletonAsyncConfiguration.addAll(module.singletonAsyncConfiguration);
      customAsyncConfiguration.addAll(module.customAsyncConfiguration);
    });
  }

  final List<MiodiModule> modules;

  T get<T>({String name = defaultProviderName}) {
    if (singletonConfiguration[name].containsKey(T))
      return singletonConfiguration[name][T].provide() as T;
    if (factoryConfiguration[name].containsKey(T))
      return factoryConfiguration[name][T].provide() as T;
    if (customConfiguration[name].containsKey(T))
      return customConfiguration[name][T].provide() as T;
    throw NotFoundProviderException(T);
  }

  Future<T> getAsync<T>({String name = defaultProviderName}) {
    if (singletonAsyncConfiguration[name].containsKey(T))
      return singletonAsyncConfiguration[name][T].provide();
    if (factoryAsyncConfiguration[name].containsKey(T))
      return factoryAsyncConfiguration[name][T].provide();
    if (customAsyncConfiguration[name].containsKey(T))
      return customAsyncConfiguration[name][T].provide();
    throw NotFoundProviderException(T);
  }
}